@@ -6,7 +6,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 use File::Copy;
 use File::Path;
@@ -22,7 +22,7 @@ our @EXPORTABLE = qw(
 cp rmf mkd cd make 
 cdback download untar 
 pie slurp blurt mv tap 
-plough qquote perm_cp
+plough qquote quote perm_cp
 sysrun untar_in pick ask
 hammer say
 sudo_me bin_find
@@ -544,6 +544,8 @@ sub slurp {
     my $data = <FILE>;
     close FILE;
 
+    DEBUG "Read ", length($data), " bytes from $file";
+
     return $data;
 }
 
@@ -562,7 +564,8 @@ sub blurt {
 ###############################################
     my($data, $file, $append) = @_;
 
-    INFO(($append ? "appending" : "writing") . " data to $file");
+    INFO(($append ? "appending" : "writing") . " " .
+         length($data) . " bytes to $file");
 
     open FILE, ">" . ($append ? ">" : "") . $file 
         or LOGDIE "Cannot open $file for writing ($!)";
@@ -579,10 +582,37 @@ Capture STDOUT and STDERR, and return them as strings. If
 C<$exit_code> is 0, the command succeeded. If it is different,
 the command failed and $exit_code holds its exit code.
 
-Please note that C<tap()> is limited to single shell commands, it
-won't work with output redirectors (C<ls E<gt>/tmp/foo>), pipes
-(C<ls | grep foo>), or commands concatenated with semicolons
-(C<ls /etc; ls /tmp>).
+Please note that C<tap()> is limited to single shell
+commands, it won't work with output redirectors (C<ls E<gt>/tmp/foo>
+2E<gt>&1).
+
+In default mode, C<tap()> will concatenate the command and args
+given and create a shell command line by redirecting STDERR to a temporary
+file. C<tap("ls", "/tmp")>, for example, will result in
+
+    'ls' '/tmp' 2>/tmp/sometempfile |
+
+Note that all commands are protected by single quotes to make sure
+arguments containing spaces are processed as singles, and no globbing
+happens on wildcards. Arguments containing single quotes or backslashes
+are escaped properly.
+
+If quoting is undesirable, C<tap()> accepts an option hash as
+its first parameter, 
+
+    tap({no_quotes => 1}, "ls", "/tmp/*");
+
+which will suppress any quoting:
+
+    ls /tmp/* 2>/tmp/sometempfile |
+
+Or, if you prefer double quotes, use
+
+    tap({double_quotes => 1}, "ls", "/tmp/$VAR");
+
+wrapping all args so that shell variables are interpolated properly:
+
+    "ls" "/tmp/$VAR" 2>/tmp/sometempfile |
 
 =cut
 
@@ -591,12 +621,26 @@ sub tap {
 ###############################################
     my(@args) = @_;
 
+    my $opts = {};
+
+    $opts = shift @args if ref $args[0] eq "HASH";
+
     my $tmpfh   = File::Temp->new(UNLINK => 1, SUFFIX => '.dat');
     my $tmpfile = $tmpfh->filename();
 
     DEBUG "tempfile $tmpfile created";
 
-    my $cmd = join ' ', map { qquote($_, ":shell") } @args;
+    my $cmd;
+
+    if($opts->{no_quotes}) {
+        $cmd = join ' ', @args;
+    } elsif($opts->{double_quotes}) {
+        $cmd = join ' ', map { qquote($_, ":shell") } @args;
+    } else {
+            # Default mode: Single quotes
+        $cmd = join ' ', map { quote($_) } @args;
+    }
+       
     $cmd = "$cmd 2>$tmpfile |";
     INFO "tapping $cmd";
 
@@ -696,6 +740,30 @@ sub qquote {
 
 =pod
 
+=item C<$quoted_string = quote($string, [$metachars])>
+
+Similar to C<qquote()>, just puts a string in single quotes.
+
+=cut
+
+###############################################
+sub quote {
+###############################################
+    my($str, $metas) = @_;
+
+    $str =~ s/([\\'])/\\$1/g;
+
+    if(defined $metas) {
+        $metas = '' if $metas eq ":shell";
+        $metas =~ s/\]/\\]/g;
+        $str =~ s/([$metas])/\\$1/g;
+    }
+
+    return "\'$str\'";
+}
+
+=pod
+
 =item C<perm_cp($src, $dst, ...)>
 
 Read the C<$src> file's user permissions and modify all
